import configparser
import base64
import os
import subprocess

from charmhelpers import fetch
from charmhelpers.core import (
    hookenv,
    host,
)

from charmhelpers.payload import archive, execd


# a list of debian packages to install:
REQUIRED_PACKAGES = ['python3', 'python-virtualenv', 'python3-jinja2']


config = hookenv.config()

BASE_DIR = '/srv/result-enum-worker/{}'.format(config['environment'])
SERVICE_DIR = os.path.join(BASE_DIR, 'result-enum-worker')
LOG_DIR = os.path.join(BASE_DIR, 'logs')


def log_start(service_name):
    hookenv.log('result-enum-worker starting')


def ensure_directories(service_name):
    host.mkdir(BASE_DIR, owner='root', group='ubuntu')


def basenode(service_name):
    hookenv.log("Executing basenode")
    execd.execd_preinstall()


def install_packages(service_name):
    hookenv.log('Installing dependencies...')
    fetch.configure_sources(update=True)
    fetch.apt_install(REQUIRED_PACKAGES, fatal=True)


def get_service_code_from_tarball(service_name):
    files_dir = os.path.join(hookenv.charm_dir(), 'files')
    tarball = os.path.join(files_dir, 'result-enum-worker.tgz')
    if not os.path.exists(SERVICE_DIR):
        hookenv.log('Installing the code for the first time from tarball')
        archive.extract_tarfile(tarball, os.path.dirname(SERVICE_DIR))


def get_config_file(service_name):
    config = hookenv.config()
    config_content = base64.b64decode(config['config-file']).decode()
    config_path = os.path.join(SERVICE_DIR, 'result-enum-worker.ini')
    parser = configparser.ConfigParser()
    parser.read_string(config_content)
    mongo_address = ''
    relations = hookenv.relations()['mongodb']
    for rel_id, info in relations.items():
        for endpoint, data in info.items():
            if endpoint.startswith('mongodb'):
                host = data['private-address']
                port = data.get('port', None)
                if port:
                    mongo_address = '%s:%s' % (host, port)
                else:
                    mongo_address = host
                break
    assert mongo_address != ''
    parser['mongodb']['server'] = mongo_address
    with open(config_path, 'w') as f:
        parser.write(f)


def prepare_for_worker(service_name):
    # add user and group
    host.adduser('result-enum-worker')
    host.add_group('result-enum-worker')
    host.add_user_to_group('result-enum-worker', 'result-enum-worker')
    # create log directory owned by the user
    host.mkdir(LOG_DIR, owner='result-enum-worker', group='result-enum-worker')


def install_python_packages(service_name):
    env_dir = os.path.join(SERVICE_DIR, 've')
    if os.path.exists(env_dir):
        hookenv.log('Service venv already exists, nothing to do ...')
        return

    hookenv.log('Installing python packages into the venv ...')
    subprocess.check_call(['virtualenv', '-p', 'python3', env_dir])

    pip_cache = os.path.join(hookenv.charm_dir(), 'files', 'pip-cache')
    requirements = os.path.join(SERVICE_DIR, 'requirements.txt')

    if not os.path.exists(requirements):
        hookenv.log("No requirements.txt file found, nothing to install")
        return

    hookenv.log('Installing from download cache.')
    subprocess.check_call(['%s/bin/pip' % env_dir,
                           'install',
                           '--no-index',
                           '--find-links={}'.format(pip_cache),
                           '-r', requirements])

    subprocess.check_call(['%s/bin/pip' % env_dir,
                           'install', '--no-deps', '-e', SERVICE_DIR])
